# Eve IDs
EVE_CATEGORY_ID_STATION = 3
EVE_CATEGORY_ID_SHIP = 6
EVE_CATEGORY_ID_MODULE = 7
EVE_CATEGORY_ID_CHARGE = 8
EVE_CATEGORY_ID_BLUEPRINT = 9
EVE_CATEGORY_ID_SKILL = 16
EVE_CATEGORY_ID_DRONE = 18
EVE_CATEGORY_ID_IMPLANT = 20
EVE_CATEGORY_ID_FIGHTER = 87
EVE_CATEGORY_ID_STRUCTURE = 65
EVE_TYPE_ID_SOLAR_SYSTEM = 5

EVE_LOCATION_FLAGS = [
    "AssetSafety",
    "AutoFit",
    "Bonus",
    "Booster",
    "BoosterBay",
    "Capsule",
    "Cargo",
    "CorpDeliveries",
    "CorpSAG1",
    "CorpSAG2",
    "CorpSAG3",
    "CorpSAG4",
    "CorpSAG5",
    "CorpSAG6",
    "CorpSAG7",
    "CrateLoot",
    "Deliveries",
    "DroneBay",
    "DustBattle",
    "DustDatabank",
    "FighterBay",
    "FighterTube0",
    "FighterTube1",
    "FighterTube2",
    "FighterTube3",
    "FighterTube4",
    "FleetHangar",
    "FrigateEscapeBay",
    "Hangar",
    "HangarAll",
    "HiSlot0",
    "HiSlot1",
    "HiSlot2",
    "HiSlot3",
    "HiSlot4",
    "HiSlot5",
    "HiSlot6",
    "HiSlot7",
    "HiddenModifiers",
    "Implant",
    "Impounded",
    "JunkyardReprocessed",
    "JunkyardTrashed",
    "LoSlot0",
    "LoSlot1",
    "LoSlot2",
    "LoSlot3",
    "LoSlot4",
    "LoSlot5",
    "LoSlot6",
    "LoSlot7",
    "Locked",
    "MedSlot0",
    "MedSlot1",
    "MedSlot2",
    "MedSlot3",
    "MedSlot4",
    "MedSlot5",
    "MedSlot6",
    "MedSlot7",
    "OfficeFolder",
    "Pilot",
    "PlanetSurface",
    "QuafeBay",
    "QuantumCoreRoom",
    "Reward",
    "RigSlot0",
    "RigSlot1",
    "RigSlot2",
    "RigSlot3",
    "RigSlot4",
    "RigSlot5",
    "RigSlot6",
    "RigSlot7",
    "SecondaryStorage",
    "ServiceSlot0",
    "ServiceSlot1",
    "ServiceSlot2",
    "ServiceSlot3",
    "ServiceSlot4",
    "ServiceSlot5",
    "ServiceSlot6",
    "ServiceSlot7",
    "ShipHangar",
    "ShipOffline",
    "Skill",
    "SkillInTraining",
    "SpecializedAmmoHold",
    "SpecializedCommandCenterHold",
    "SpecializedFuelBay",
    "SpecializedGasHold",
    "SpecializedIndustrialShipHold",
    "SpecializedLargeShipHold",
    "SpecializedMaterialBay",
    "SpecializedMediumShipHold",
    "SpecializedMineralHold",
    "SpecializedOreHold",
    "SpecializedPlanetaryCommoditiesHold",
    "SpecializedSalvageHold",
    "SpecializedShipHold",
    "SpecializedSmallShipHold",
    "StructureActive",
    "StructureFuel",
    "StructureInactive",
    "StructureOffline",
    "SubSystemBay",
    "SubSystemSlot0",
    "SubSystemSlot1",
    "SubSystemSlot2",
    "SubSystemSlot3",
    "SubSystemSlot4",
    "SubSystemSlot5",
    "SubSystemSlot6",
    "SubSystemSlot7",
    "Unlocked",
    "Wallet",
    "Wardrobe",
]
